--========================================================--
--                EBFM WorldQuest                         --
--                                                        --
-- Author      :  kurapica125@outlook.com                 --
-- Create Date :  2017/03/07                              --
--========================================================--

--========================================================--
Scorpio     "EnhanceBattlefieldMinimap.WorldQuest"   "1.0.0"
--========================================================--

----------------------------------------------
-------------- Module Event Handler ----------
----------------------------------------------
function OnLoad(self)
    _SVDB:SetDefault { ShowWorldQuest = true }

    TaskPOIArray = List()
end

__SlashCmd__("ebfm", "worldquest", _Locale["on/off - show world quest marks"])
function ToggleWorldQuest(opt)
    if opt == "on" then
        if not _SVDB.ShowWorldQuest then
            _SVDB.ShowWorldQuest = true
            EBFM_REFRESH()
        end
    elseif opt == "off" then
        if _SVDB.ShowWorldQuest then
            _SVDB.ShowWorldQuest = false
            TaskPOIArray:Each("x=>x:Hide()")
        end
    else
        return false
    end
end

----------------------------------------------
-------------- System Event Handler ----------
----------------------------------------------
__SystemEvent__() __Thread__()
function EBFM_REFRESH()
    if not _SVDB.ShowWorldQuest then return end

    Next()

    local mapAreaID     = GetCurrentMapAreaID()
    local taskInfo      = C_TaskQuest.GetQuestsForPlayerByMapID(mapAreaID)
    local numTaskPOIs   = taskInfo and #taskInfo or 0

    local hasWorldQuests= false
    local taskIconIndex = 1

    if ( numTaskPOIs > 0 ) then
        for i, info in ipairs(taskInfo) do
            if ( HaveQuestData(info.questId) ) then
                local taskPOI
                local isWorldQuest = QuestUtils_IsQuestWorldQuest(info.questId)
                if ( isWorldQuest ) then
                    taskPOI = TryCreatingWorldQuestPOI(info, taskIconIndex)
                else
                    taskPOI = TryCreatingBonusObjectivePOI(info, taskIconIndex)
                end

                if ( taskPOI ) then
                    AddPOIButton(taskPOI, info.x, info.y, isWorldQuest and WORLD_MAP_POI_FRAME_LEVEL_OFFSETS.WORLD_QUEST or WORLD_MAP_POI_FRAME_LEVEL_OFFSETS.BONUS_OBJECTIVE)
                    taskPOI.questID = info.questId
                    taskPOI.numObjectives = info.numObjectives
                    taskPOI:Show()

                    taskIconIndex = taskIconIndex + 1

                    if ( isWorldQuest ) then
                        hasWorldQuests = true
                    end
                end
            end
        end
    end

    if taskInfo then wipe(taskInfo) taskInfo = nil end

    -- Hide unused icons in the pool
    for i = taskIconIndex, #TaskPOIArray do
        TaskPOIArray[i]:Hide()
    end
end

function GetTaskPOI(index)
    local button = TaskPOIArray[index]

    while #TaskPOIArray < index do
        button = CreateFrame("Button", nil, BattlefieldMinimapPOI)
        button:SetFlattensRenderLayers(true)
        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:SetScript("OnEnter", HijackTaskPOI_OnEnter)
        button:SetScript("OnLeave", TaskPOI_OnLeave)
        button:SetScript("OnClick", TaskPOI_OnClick)

        button.UpdateTooltip = HijackTaskPOI_OnEnter

        button.Texture = button:CreateTexture(nil, "BACKGROUND")

        button.Glow = button:CreateTexture(nil, "BACKGROUND", -2)
        button.Glow:SetSize(50, 50)
        button.Glow:SetPoint("CENTER")
        button.Glow:SetTexture("Interface/WorldMap/UI-QuestPoi-IconGlow.tga")
        button.Glow:SetBlendMode("ADD")

        button.SelectedGlow = button:CreateTexture(nil, "OVERLAY", 2)
        button.SelectedGlow:SetBlendMode("ADD")

        button.CriteriaMatchGlow = button:CreateTexture(nil, "BACKGROUND", -1)
        button.CriteriaMatchGlow:SetAlpha(.6)
        button.CriteriaMatchGlow:SetBlendMode("ADD")

        button.SpellTargetGlow = button:CreateTexture(nil, "OVERLAY", 1)
        button.SpellTargetGlow:SetAtlas("worldquest-questmarker-abilityhighlight", true)
        button.SpellTargetGlow:SetAlpha(.6)
        button.SpellTargetGlow:SetBlendMode("ADD")
        button.SpellTargetGlow:SetPoint("CENTER", 0, 0)

        button.Underlay = button:CreateTexture(nil, "BACKGROUND")
        button.Underlay:SetWidth(34)
        button.Underlay:SetHeight(34)
        button.Underlay:SetPoint("CENTER", 0, -1)

        button.TimeLowFrame = CreateFrame("Frame", nil, button)
        button.TimeLowFrame:SetSize(22, 22)
        button.TimeLowFrame:SetPoint("CENTER", -10, -10)
        button.TimeLowFrame.Texture = button.TimeLowFrame:CreateTexture(nil, "OVERLAY")
        button.TimeLowFrame.Texture:SetAllPoints(button.TimeLowFrame)
        button.TimeLowFrame.Texture:SetAtlas("worldquest-icon-clock")

        WorldMap_ResetPOI(button, true, false)

        TaskPOIArray:Insert(button)
    end

    return button
end

function TryCreatingWorldQuestPOI(info, taskIconIndex)
    if ( WorldMap_IsWorldQuestSuppressed(info.questId) or not WorldMap_DoesWorldQuestInfoPassFilters(info) ) then
        return nil
    end

    local tagID, tagName, worldQuestType, rarity, isElite, tradeskillLineIndex = GetQuestTagInfo(info.questId)

    local taskPOI = GetTaskPOI(taskIconIndex)
    local selected = info.questId == GetSuperTrackedQuestID()

    local isCriteria = WorldMapFrame.UIElementsFrame.BountyBoard:IsWorldQuestCriteriaForSelectedBounty(info.questId)
    local isSpellTarget = SpellCanTargetQuest() and IsQuestIDValidSpellTarget(info.questId)

    taskPOI.worldQuest = true
    taskPOI.Texture:SetDrawLayer("OVERLAY")

    WorldMap_SetupWorldQuestButton(taskPOI, worldQuestType, rarity, isElite, tradeskillLineIndex, info.inProgress, selected, isCriteria, isSpellTarget)

    C_TaskQuest.RequestPreloadRewardData(info.questId)

    return taskPOI
end

function TryCreatingBonusObjectivePOI(info, taskIconIndex)
    local taskPOI = GetTaskPOI(taskIconIndex)

    taskPOI:SetSize(24, 24)
    taskPOI:SetNormalTexture(nil)
    taskPOI:SetPushedTexture(nil)
    taskPOI:SetHighlightTexture(nil)
    taskPOI.Underlay:Hide()
    taskPOI.Texture:SetAtlas("QuestBonusObjective")
    taskPOI.Texture:SetSize(24, 24)
    taskPOI.Texture:SetDrawLayer("BACKGROUND")
    taskPOI.TimeLowFrame:Hide()
    taskPOI.CriteriaMatchGlow:Hide()
    taskPOI.SpellTargetGlow:Hide()
    taskPOI.Glow:Hide()
    taskPOI.SelectedGlow:Hide()
    taskPOI.worldQuest = false

    return taskPOI
end

function HijackTaskPOI_OnEnter(self)
    WorldMap_HijackTooltip(BattlefieldMinimapScroll)
    return TaskPOI_OnEnter(self)
end
